import org.xml.sax.*;
import org.w3c.dom.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import java.util.ArrayList;
import net.sf.saxon.Configuration;
import net.sf.saxon.lib.NamespaceConstant;
import net.sf.saxon.om.DocumentInfo;
import net.sf.saxon.om.NodeInfo;
import net.sf.saxon.xpath.XPathFactoryImpl;
import org.xml.sax.InputSource;

import javax.xml.namespace.QName;
import javax.xml.transform.sax.SAXSource;
import javax.xml.xpath.*;
import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.util.Iterator;
import java.util.List;
import net.sf.saxon.type.Type;


public class XPathExample {


    public static void main (String args[])   {

        if (args.length != 2) {
            System.err.println("Usage: java XPathEvalSaxon input-file '/xpath/expression'");
            System.exit(1);
        }

	try {

	  System.setProperty("javax.xml.xpath.XPathFactory:"+NamespaceConstant.OBJECT_MODEL_SAXON,
			   "net.sf.saxon.xpath.XPathFactoryImpl");

	  XPathFactory xpf = XPathFactory.newInstance(NamespaceConstant.OBJECT_MODEL_SAXON);

	  XPath xpe = xpf.newXPath();

	  System.err.println("Loaded XPath Provider " + xpe.getClass().getName());

	  long startTime = System.nanoTime ();
	  InputSource is = new InputSource(args[0]);
	  SAXSource ss = new SAXSource(is);
	  Configuration config = ((XPathFactoryImpl) xpf).getConfiguration();
	  NodeInfo doc = config.buildDocumentTree(ss).getRootNode();
	  long stopTime = System.nanoTime ();
	  System.err.println ("Loading document : " + (stopTime - startTime) / 1000000 + "ms");

	  startTime = System.nanoTime ();
	  XPathExpression cexpr = xpe.compile(args[1]);
	  stopTime = System.nanoTime ();
	  System.err.println ("Compiling XPath expression : " + (stopTime - startTime) / 1000000 + "ms");

	  startTime = System.nanoTime ();
	  @SuppressWarnings("unchecked")
	    ArrayList<NodeInfo> nodes = (ArrayList<NodeInfo>) cexpr.evaluate(doc, XPathConstants.NODESET);
	  stopTime = System.nanoTime ();
	  long firstRun = (stopTime - startTime);
	  System.err.println ("Evaluating XPath expression (1st run): " + firstRun / 1000000 + "ms");

	  startTime = System.nanoTime ();
	  for (int i = 1; i < 10; i++) {

	    cexpr.evaluate(doc, XPathConstants.NODESET);

	  };

	  stopTime = System.nanoTime ();
	  System.err.println ("Evaluating XPath expression (average 10 runs): " + (stopTime - startTime + firstRun) / (10 * 1000000) + "ms");

	  Transformer serializer = TransformerFactory.newInstance().newTransformer();
	  serializer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
	  StreamResult o = new StreamResult(System.out);
	  startTime = System.nanoTime();
	  System.out.println("<xml_result num=\"1\">");
	  for(int i = 0; i < nodes.size(); i++){
	    NodeInfo n = nodes.get(i);
	    switch (n.getNodeKind()) {
	    case Type.ATTRIBUTE:
	      System.out.print (n.getLocalPart() + "=" + n.getStringValue());
	      break;
	    default:
	      serializer.transform(n, o);
	      break;
	    };
	    System.out.println();
	  };
	  System.out.println("</xml_result>");
	  stopTime = System.nanoTime();
	  System.err.println ("Serializing document : " + (stopTime - startTime) / 1000000 + "ms");

	} catch (XPathException e) {
	  System.out.println (e.getCause());
	} catch (Exception e) {
	  System.out.println(e);
	};

    }

}
